@@ -1,6 +1,6 @@
 /* 
    sitecopy, for managing remote web sites.
-   Copyright (C) 1998-2000, Joe Orton <joe@orton.demon.co.uk>
+   Copyright (C) 1998-2003, Joe Orton <joe@manyfish.co.uk>
                                                                      
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,6 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-   $Id: rcfile.c,v 1.6 2001/01/28 02:58:40 davek Exp $
 */
 
 #include <config.h>
@@ -40,7 +39,7 @@
 #include <unistd.h>
 #endif /* HAVE_UNISTD_H */
 
-#include <string_utils.h>
+#include <ne_string.h>
 #include <ne_alloc.h>
 
 #include "common.h"
@@ -62,8 +61,8 @@ char *rcfile_err;
 /** Not quite so global variables **/
 
 /* These are appended to $HOME */
-#define RCNAME "/." PACKAGE "rc"
-#define COPYNAME "/." PACKAGE "/"
+#define RCNAME "/.sitecopyrc"
+#define COPYNAME "/.sitecopy/"
 #define NETRCNAME "/.netrc"
 
 /* Stores the list of entries in the ~/.netrc */
@@ -109,7 +108,10 @@ int rcfile_read(struct site **sites)
     default_site.perms = sitep_ignore;
     default_site.symlinks = sitesym_follow;
     default_site.protocol = siteproto_ftp;
+    default_site.proto_string = ne_strdup("ftp");
+
     default_site.ftp_pasv_mode = true;
+    default_site.ftp_use_cwd = false;
     
     last_site = this_site = NULL;
     rcfile_linenum = 0;
@@ -218,7 +220,7 @@ int rcfile_read(struct site **sites)
 	    }
 	}
 	
-	DEBUG(DEBUG_RCFILE, "Key [%s] Value: [%s] Value2: [%s]\n", key, val, val2);
+	NE_DEBUG(DEBUG_RCFILE, "Key [%s] Value: [%s] Value2: [%s]\n", key, val, val2);
 	
 	if (strlen(key) == 0) {
 	    continue;
@@ -228,7 +230,7 @@ int rcfile_read(struct site **sites)
 	    if (this_site == NULL) {
 		if (strcmp(key, "default") == 0) {
 		    /* Setting up the default site */
-		    DEBUG(DEBUG_RCFILE, "Default site entry:\n");
+		    NE_DEBUG(DEBUG_RCFILE, "Default site entry:\n");
 		    this_site = &default_site;
 		} else {
 		    /* Need to be in a site! */
@@ -271,8 +273,10 @@ int rcfile_read(struct site **sites)
 		}		
 		this_site->name = ne_strdup(val);
 		this_site->files = NULL;
+		this_site->proto_string = ne_strdup(default_site.proto_string);
 		/* Now work out the info filename */
-		CONCAT2(this_site->infofile, copypath, val);
+		this_site->infofile = ne_concat(copypath, val, NULL);
+                this_site->certfile = ne_concat(copypath, val, ".crt", NULL);
 	    } else if (this_site == NULL) {
 		ret = RC_CORRUPT;
 	    } else if (strcmp(key, "username") == 0) {
@@ -383,12 +387,17 @@ int rcfile_read(struct site **sites)
 		} else {
 		    this_site->protocol = siteproto_unknown;
 		}
+		free(this_site->proto_string);
 		this_site->proto_string = ne_strdup(val);
 	    } else if (strcmp(key, "ftp") == 0) {
 		if (strcmp(val, "nopasv") == 0) {
 		    this_site->ftp_pasv_mode = false;
 		} else if (strcmp(val, "showquit") == 0) {
 		    this_site->ftp_echo_quit = true;		    
+		} else if (strcmp(val, "usecwd") == 0) {
+		    this_site->ftp_use_cwd = true;		    
+		} else if (strcmp(val, "nousecwd") == 0) {
+		    this_site->ftp_use_cwd = false;		    
 		} else {
 		    ret = RC_CORRUPT;
 		}
@@ -399,7 +408,9 @@ int rcfile_read(struct site **sites)
 		    this_site->http_limit = true;
 		} else if (strcmp(val, "secure") == 0) {
 		    this_site->http_secure = true;
-		} else {
+		} else if (strcmp(val, "tolerant") == 0) {
+		    this_site->http_tolerant = true;
+		} else {		    
 		    ret = RC_CORRUPT;
 		}
 	    } else if (strcmp(key, "rsh") == 0) {
@@ -421,7 +432,9 @@ int rcfile_read(struct site **sites)
 		} else {
 		    ret = RC_CORRUPT;
 		}
-	    } else {
+	    } else if (strcmp(key, "charset") == 0) {
+                NE_DEBUG(DEBUG_RCFILE, "Ignored key %s\n", key);
+            } else {
 		/* Unknown key! */
 		ret = RC_CORRUPT;
 	    }
@@ -497,8 +510,8 @@ int rcfile_verify(struct site *any_site)
 	return SITE_UNSUPPORTED;
 #endif /* USE_RSH */
     case siteproto_local:
-	    any_site->driver = &local_driver;
-	    break;
+	any_site->driver = &local_driver;
+	break;
     case siteproto_unknown:
 	return SITE_UNSUPPORTED;
     }
@@ -516,15 +529,15 @@ int rcfile_verify(struct site *any_site)
     if (any_site->server.username != NULL && any_site->server.password == NULL) {
 	if (havenetrc) {
 	    const char *pass;
-	    DEBUG(DEBUG_RCFILE, "Checking netrc for password for %s@%s...",
+	    NE_DEBUG(DEBUG_RCFILE, "Checking netrc for password for %s@%s...",
 		   any_site->server.username, any_site->server.hostname);
 	    pass = rc_get_netrc_password(any_site->server.hostname, 
 					  any_site->server.username);
 	    if (pass != NULL) {
-		DEBUG(DEBUG_RCFILE, "found!\n");
+		NE_DEBUG(DEBUG_RCFILE, "found!\n");
 		any_site->server.password = (char *) pass;
 	    } else {
-		DEBUG(DEBUG_RCFILE, "none found.\n");
+		NE_DEBUG(DEBUG_RCFILE, "none found.\n");
 	    }
 	}
     }
@@ -554,7 +567,8 @@ int rcfile_verify(struct site *any_site)
     }
     if (any_site->local_isrel) {
 	/* We skip the first char ('~') of l_r_u */
-	CONCAT2(any_site->local_root, home, any_site->local_root_user + 1);
+	any_site->local_root = ne_concat(home, any_site->local_root_user + 1,
+					 NULL);
     } else {
 	any_site->local_root = any_site->local_root_user;
     }
@@ -562,7 +576,7 @@ int rcfile_verify(struct site *any_site)
     /* Now check the local directory actually exists.
      * To do this, stat `/the/local/root/.', which will fail if the
      * can't read the directory or if it's a file not a directory */
-    CONCAT2(temp, any_site->local_root, ".");
+    temp = ne_concat(any_site->local_root, ".", NULL);
     ret = stat(temp, &localst);
     free(temp);
     if (ret != 0) {
@@ -571,15 +585,15 @@ int rcfile_verify(struct site *any_site)
 
     /* Assign default ports if they didn't bother to */
     if (any_site->server.port == 0) {
-	DEBUG(DEBUG_RCFILE, "Lookup up default port:\n");
+	NE_DEBUG(DEBUG_RCFILE, "Lookup up default port:\n");
 	any_site->server.port = (*any_site->driver->get_server_port)(any_site);
-	DEBUG(DEBUG_RCFILE, "Using port: %d\n", any_site->server.port);
+	NE_DEBUG(DEBUG_RCFILE, "Using port: %d\n", any_site->server.port);
     }
 
     if (any_site->proxy.port == 0) {
-	DEBUG(DEBUG_RCFILE, "Lookup default proxy port...\n");
+	NE_DEBUG(DEBUG_RCFILE, "Lookup default proxy port...\n");
 	any_site->proxy.port = (*any_site->driver->get_proxy_port)(any_site);
-	DEBUG(DEBUG_RCFILE, "Using port %d\n", any_site->proxy.port);
+	NE_DEBUG(DEBUG_RCFILE, "Using port %d\n", any_site->proxy.port);
     }
 
     /* TODO: ditto for proxy server */
@@ -599,10 +613,11 @@ int init_netrc() {
 }
 
 /* Checks the perms of the rcfile and site storage directory. */
-int init_paths() {
+int init_paths()
+{
     struct stat st;
     if (stat(rcfile, &st) < 0) {
-	DEBUG(DEBUG_RCFILE, "stat failed on %s: %s\n", 
+	NE_DEBUG(DEBUG_RCFILE, "stat failed on %s: %s\n", 
 	       rcfile, strerror(errno));
 	return RC_OPENFILE;
     }
@@ -621,12 +636,12 @@ int init_paths() {
 	havenetrc = true;
     }
     if (stat(copypath, &st) < 0) {
-	DEBUG(DEBUG_RCFILE, "stat failed on %s: %s\n", 
+	NE_DEBUG(DEBUG_RCFILE, "stat failed on %s: %s\n", 
 	       copypath, strerror(errno));
 	return RC_DIROPEN;
     }
 #if !defined (__EMX__) && !defined(__CYGWIN__)
-    if ((st.st_mode & ~(S_IFDIR | S_IREAD | S_IWRITE | S_IEXEC)) > 0) {
+    if (st.st_mode & (S_IRWXG | S_IRWXO)) {
 	return RC_DIRPERMS;
     }
 #endif
@@ -646,12 +661,12 @@ int init_env() {
 	}
     }
     if (rcfile == NULL) {
-	CONCAT2(rcfile, home, RCNAME);
+	rcfile = ne_concat(home, RCNAME, NULL);
     }
     if (copypath == NULL) {
-	CONCAT2(copypath, home, COPYNAME);
+	copypath = ne_concat(home, COPYNAME, NULL);
     }
-    CONCAT2(netrcfile, home, NETRCNAME);
+    netrcfile = ne_concat(home, NETRCNAME, NULL);
     return 0;
 }
 
@@ -710,8 +725,7 @@ int rcfile_write (char *filename, struct site *list_of_sites)
 	    return RC_CORRUPT;
 	}
 	
-	if (fprintf (fp, "  protocol %s\n", 
-		     current->protocol==siteproto_ftp?"ftp":"http") == -1) {
+	if (fprintf (fp, "  protocol %s\n", current->proto_string) == -1) {
 	    return RC_CORRUPT;
 	}
 	
@@ -743,6 +757,7 @@ int rcfile_write (char *filename, struct site *list_of_sites)
 	
 	RCWRITEBOOL(!current->ftp_pasv_mode, "ftp nopasv");
 	RCWRITEBOOL(current->ftp_echo_quit, "ftp showquit");
+	RCWRITEBOOL(current->ftp_use_cwd, "ftp usecwd");
 	RCWRITEBOOL(current->http_limit, "http limit");
 	RCWRITEBOOL(current->http_use_expect, "http expect");
 	
