@@ -1,6 +1,6 @@
 /* 
    sitecopy, for managing remote web sites.
-   Copyright (C) 1998-2000, Joe Orton <joe@orton.demon.co.uk>
+   Copyright (C) 1998-2002, Joe Orton <joe@manyfish.co.uk>
                                                                      
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,6 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-   $Id: sites.c,v 1.6 2001/02/21 20:45:04 davek Exp $
 */
 
 /* This is the core functionality of sitecopy, performing updates
@@ -47,16 +46,17 @@
 #include <time.h>
 #include <utime.h>
 
+/* neon */
+#include <ne_string.h>
+#include <ne_alloc.h>
+#include <ne_md5.h>
+#include <ne_socket.h>
+
 #ifdef HAVE_SNPRINTF_H
 #include "snprintf.h"
 #endif /* !HAVE_SNPRINTF_H */
 
-/* neon */
-#include <basename.h>
-#include <string_utils.h>
-#include <ne_alloc.h>
-#include <neon_md5.h>
-#include <nsocket.h>
+#include "basename.h"
 
 #include "i18n.h"
 #include "common.h"
@@ -75,35 +75,6 @@ static int proto_init(struct site *site, void **session);
 static void proto_finish(struct site *site, void *session);
 static void proto_seterror(struct site *site, void *session);
 
-/* Prototypes */
-
-static char *temp_upload_filename(const char *filename, struct site *site);
-
-static int synch_create_directories(struct site *site);
-static int synch_files(struct site *site, void *session);
-static int synch_delete_directories(struct site *site);
-
-static int update_create_directories(struct site *site, void *session);
-static int update_delete_directories(struct site *site, void *session);
-static int update_delete_files(struct site *site, void *session);
-static int update_files(struct site *site, void *session);
-static int update_move_files(struct site *site, void *session);
-static int update_links(struct site *the_site, void *session);
-
-static void file_retrieve_server(struct site_file *file, struct site *site,
-				 void *session);
-
-static void site_fetch_walk(struct site *site, struct proto_file *files);
-static void site_fetch_csum_read(void *userdata, const char *s, const size_t len);
-static int site_fetch_checksum(struct proto_file *files, struct site *site,
-			       void *session);
-
-static void site_flatlist_items(FILE *f, struct site *site, 
-				const enum file_diff diff, const char *name);
-
-static int 
-site_verify_compare(struct site *site, const struct proto_file *files);
-
 struct site *site_find(const char *sitename)
 {
     struct site *current;
@@ -190,7 +161,6 @@ static int synch_files(struct site *site, void *session)
 				    full_local, strerror(errno));
 		    }
 		}
-		file_downloaded(current, site);
 		if (file_perms_changed(current, site)) {
 		    fe_setting_perms(current);
 		    if (chmod(full_local, current->stored.mode) < 0) {
@@ -199,6 +169,8 @@ static int synch_files(struct site *site, void *session)
 			fe_set_perms(current, true, NULL);
 		    }
 		}
+		/* TODO: not strictly true if the chmod failed. */
+		file_downloaded(current, site);
 	    }
 	    free(full_local);
 	    free(full_remote);
@@ -340,7 +312,7 @@ file_retrieve_server(struct site_file *file, struct site *site, void *session)
     free(full_remote);
 }
 
-int update_create_directories(struct site *site, void *session)
+static int update_create_directories(struct site *site, void *session)
 {
     struct site_file *current;
     int ret = 0;
@@ -391,7 +363,7 @@ static char *temp_upload_filename(const char *filename, struct site *site)
     return ret;
 }
 		
-int update_delete_files(struct site *site, void *session)
+static int update_delete_files(struct site *site, void *session)
 {
     struct site_file *current, *next;
     int ret = 0;
@@ -679,11 +651,10 @@ static void proto_seterror(struct site *site, void *session)
 
 const char *site_get_protoname(struct site *site) 
 {
-    if (site->protocol == siteproto_unknown) {
-	return "Unknown";
-    } else {
+    if (site->driver)
 	return site->driver->protocol_name;
-    }
+    else
+	return site->proto_string;
 }
 
 static int proto_init(struct site *site, void **session)
@@ -763,7 +734,7 @@ int site_readfiles(struct site *site)
     int ret;
     site_destroy(site);
     ret = site_read_stored_state(site);
-    if (ret == SITE_OK) {
+    if (ret >= SITE_OK) {
 	site_read_local_state(site);
     }
     return ret;
@@ -798,7 +769,7 @@ void site_read_local_state(struct site *site)
 	/* Pop the stack */
 	this = dirstack[--dirtop];
 	
-	DEBUG(DEBUG_FILES, "Scanning: %s\n", this);
+	NE_DEBUG(DEBUG_FILES, "Scanning: %s\n", this);
 	curdir = opendir(this);
 	if (curdir == NULL) {
 	    fe_warning("Could not read directory", this, strerror(errno));
@@ -824,7 +795,7 @@ void site_read_local_state(struct site *site)
 	    
 	    if (full != NULL) free(full);
 
-	    CONCAT2(full, this, ent->d_name);
+	    full = ne_concat(this, ent->d_name, NULL);
 
 #ifdef __EMX__
 /* There are no symlinks under OS/2, use stat() instead */
@@ -841,21 +812,21 @@ void site_read_local_state(struct site *site)
 #ifndef __EMX__
 	    /* Is this a symlink? */
 	    if (S_ISLNK(item.st_mode)) {
-		DEBUG(DEBUG_FILES, "symlink - ");
+		NE_DEBUG(DEBUG_FILES, "symlink - ");
 		if (site->symlinks == sitesym_ignore) {
 		    /* Just skip it */
-		    DEBUG(DEBUG_FILES, "ignoring.\n");
+		    NE_DEBUG(DEBUG_FILES, "ignoring.\n");
 		    continue;
 		} else if (site->symlinks == sitesym_follow) {
-		    DEBUG(DEBUG_FILES, "followed - ");
+		    NE_DEBUG(DEBUG_FILES, "followed - ");
 		    /* Else, carry on as normal, stat the real file */
 		    if (stat(full, &item) == -1) {
 			/* It's probably a broken link */
-			DEBUG(DEBUG_FILES, "broken.\n");
+			NE_DEBUG(DEBUG_FILES, "broken.\n");
 			continue;
 		    }
 		} else {
-		    DEBUG(DEBUG_FILES, "maintained:\n");
+		    NE_DEBUG(DEBUG_FILES, "maintained:\n");
 		}
 	    }
 #endif /* __EMX__ */
@@ -890,7 +861,7 @@ void site_read_local_state(struct site *site)
 	    else if (S_ISLNK(item.st_mode)) {
 		char tmp[BUFSIZ] = {0};
 		type = file_link;
-		DEBUG(DEBUG_FILES, "symlink being maintained.\n");
+		NE_DEBUG(DEBUG_FILES, "symlink being maintained.\n");
 		if (readlink(full, tmp, BUFSIZ) == -1) {
 		    fe_warning(_("The target of the symlink could not be read."), full, strerror(errno));
 		    continue;
@@ -906,10 +877,10 @@ void site_read_local_state(struct site *site)
 		    dirstack = realloc(dirstack, sizeof(char *) * dirmax);
 		}
 		/* Add it to the search stack */
-		CONCAT2(dirstack[dirtop], full, "/");
+		dirstack[dirtop] = ne_concat(full, "/", NULL);
 		dirtop++;
 	    } else {
-		DEBUG(DEBUG_FILES, "something else.\n");
+		NE_DEBUG(DEBUG_FILES, "something else.\n");
 		continue;
 	    }
 	    
@@ -968,6 +939,45 @@ void site_initialize(struct site *site)
     site_destroy_stored(site);
 }
 
+/* Munge modtimes of 'file' accordingly; when modtime of file on
+ * server is 'remote_mtime'. */
+static void munge_modtime(struct site_file *file, time_t remote_mtime,
+			  struct site *site)
+{
+    /* If this is a file, and we are using timesize mode, and we have
+     * a local copy of this file already, we have to cope with the
+     * modtimes problem.  The problem is that the modtime locally will
+     * ALWAYS be different from the modtime on the SERVER.  */
+    if (file->type == file_file && site->state_method == state_timesize) {
+	if (file->local.exists) {
+	    /* If we are in safe mode, we can actually check whether
+	     * the remote file has changed or not when we are using
+	     * timesize mode, by comparing what we thought the server
+	     * modtime was with what the actual (fetched) server
+	     * modtime is. Got that? */
+	    NE_DEBUG(DEBUG_FILES, "Fetch: %ld vs %ld\n", 
+		     file->server.time, remote_mtime);
+	    if (site->safemode && file->server.exists &&
+		file->server.time != remote_mtime) {
+		NE_DEBUG(DEBUG_FILES, 
+			 "Fetch: Marking changed file changed.\n");
+		file->stored.time = file->local.time + 1;
+	    } else {
+		NE_DEBUG(DEBUG_FILES, "Fetch: Marking unchanged files same.\n");
+		file->stored.time = file->local.time;
+	    }
+	} else {
+	    /* If the local file doesn't exist, pretend the file was
+	     * last uploaded "now" (an arbitrary time is adequate, but
+	     * "now" is the least confusing). */
+	    file->stored.time = time(NULL);
+	}
+
+	/* update the diff. */
+	file_set_diff(file, site);
+    }
+}
+
 /* This walks the files list as returned by the protocol driver,
  * and converts it into a 'real' files list. */
 static void 
@@ -1005,40 +1015,13 @@ site_fetch_walk(struct site *site, struct proto_file *files)
 	    file = file_set_stored(type, &state, site);
 	    fe_fetch_found(file);
 
-	    /* If this is a file, and we are using timesize mode,
-	     * and we have a local copy of this file already, we
-	     * have to cope with the modtimes problem.  The problem
-	     * is that the modtime locally will ALWAYS be different
-	     * from the modtime on the SERVER.
-	     */
-	    if (file->type == file_file && 
-		file->local.exists && 
-		site->state_method == state_timesize) {
-		/* If we are in safe mode, we can actually check
-		 * whether the remote file has changed or not when we
-		 * are using timesize mode, by comparing what we
-		 * thought the server modtime was with what the actual
-		 * (fetched) server modtime is. Got that? */
-		DEBUG(DEBUG_FILES, "Fetch: %ld vs %ld\n",
-		      file->server.time, state.time);
-		if (site->safemode && file->server.exists &&
-		    file->server.time != state.time) {
-		    DEBUG(DEBUG_FILES, 
-			  "Fetch: Marking changed file changed.\n");
-		    file->stored.time = file->local.time + 1;
-		} else {
-		    DEBUG(DEBUG_FILES,
-			  "Fetch: Marking unchanged files same.\n");
-		    file->stored.time = file->local.time;
-		}
-		file_set_diff(file, site);
-	    }
+	    munge_modtime(file, this_file->modtime, site);
 
 	    if (site->safemode) {
-		/* Now store the server modtime. */
+		/* Store the server modtime. */
 		file->server.time = this_file->modtime;
 		file->server.exists = true;
-	    }	    
+	    }
 
 	} else {
 	    free(this_file->filename);
@@ -1054,15 +1037,15 @@ site_fetch_walk(struct site *site, struct proto_file *files)
 static void
 site_fetch_csum_read(void *userdata, const char *s, size_t len)
 {
-    struct md5_ctx *md5 = userdata;
-    md5_process_bytes(s, len, md5);
+    struct ne_md5_ctx *md5 = userdata;
+    ne_md5_process_bytes(s, len, md5);
 }
 
 /* Retrieve the remote checksum for all files */
 static int 
 site_fetch_checksum(struct proto_file *files, struct site *site, void *session)
 {
-    struct md5_ctx md5;
+    struct ne_md5_ctx md5;
     struct proto_file *file;
     int ret = 0;
     
@@ -1070,15 +1053,15 @@ site_fetch_checksum(struct proto_file *files, struct site *site, void *session)
 	if (file->type == proto_file && 
 	    !file_isexcluded(file->filename, site)) {
 	    char *full_remote;
-	    CONCAT2(full_remote, site->remote_root, file->filename);
-	    md5_init_ctx(&md5);
+	    full_remote = ne_concat(site->remote_root, file->filename, NULL);
+	    ne_md5_init_ctx(&md5);
 	    fe_checksumming(file->filename);
 	    if (CALL(file_read)(session, full_remote, 
 				site_fetch_csum_read, &md5) != SITE_OK) {
 		ret = 1;
 		fe_checksummed(full_remote, false, DRIVER_ERR);
 	    } else {
-		md5_finish_ctx(&md5, file->checksum);
+		ne_md5_finish_ctx(&md5, file->checksum);
 		fe_checksummed(full_remote, true, NULL);
 	    }
 	    free(full_remote);
@@ -1107,10 +1090,11 @@ int site_fetch(struct site *site)
 	return SITE_UNSUPPORTED;
     }
 
-    ret = CALL(fetch_list)(session, site->remote_root, 
-			    site->safemode, &files);
-
-    if (site->state_method == state_checksum) {
+    /* we want modtimes if in safe mode, or in timesize mode. */
+    ret = CALL(fetch_list)(session, site->remote_root, site->safemode || 
+			   site->state_method == state_timesize, &files);
+    
+    if (ret == SITE_OK && site->state_method == state_checksum) {
 	site_fetch_checksum(files, site, session);
     }
     
@@ -1133,11 +1117,14 @@ int site_fetch(struct site *site)
  * If we had a generic file_set this would be easy and clean and spot
  * moved files too. We need a generic file_set.
  */
-int site_verify_compare(struct site *site, const struct proto_file *files)
+static int site_verify_compare(struct site *site, 
+			       const struct proto_file *files,
+			       int *numremoved)
 {
     struct site_file *file;
     const struct proto_file *lfile;
     int numremote = 0;
+
     /* Clear live state */
     for_each_file(file, site) {
 	if (file->stored.exists) {
@@ -1146,26 +1133,37 @@ int site_verify_compare(struct site *site, const struct proto_file *files)
     }
 
     for (lfile = files; lfile != NULL; lfile = lfile->next) {
+	enum file_diff diff = file_new;
+
 	numremote--;
 	for_each_file(file, site) {
 	    if (file->stored.exists &&
 		(strcmp(file->stored.filename, lfile->filename) == 0)) {
 		/* Do a mini file_compare job */
+		diff = file_unchanged;
 		if (site->state_method == state_checksum) {
 		    if (memcmp(file->stored.checksum, lfile->checksum, 16))
-			return SITE_ERRORS;
+			diff = file_changed;
 		} else {
 		    if ((file->stored.size != lfile->size) ||
 			(site->safemode && 
 			 (file->server.time != lfile->modtime))) {
-			return SITE_ERRORS;
+			diff = file_changed;
 		    }
 		}
 		break;
 	    }
 	}
+	
+	/* If new files were added, adjust the count */
+	if (diff == file_new)
+	    numremote++;
+
+	fe_verified(lfile->filename, diff);	
     }
-    
+
+    *numremoved = numremote;
+
     if (numremote != 0) {
 	return SITE_ERRORS;
     } else {
@@ -1177,7 +1175,7 @@ int site_verify_compare(struct site *site, const struct proto_file *files)
 /* Compares what's on the server with what we THINK is on the server.
  * Returns SITE_OK if match, SITE_ERRORS if doesn't match.
  */
-int site_verify(struct site *site)
+int site_verify(struct site *site, int *numremoved)
 {
     struct proto_file *files = NULL;
     void *session;
@@ -1201,7 +1199,7 @@ int site_verify(struct site *site)
     
     if (ret == SITE_OK) {
 	/* Return whether they matched or not */
-	return site_verify_compare(site, files);
+	return site_verify_compare(site, files, numremoved);
     } else {
 	return SITE_FAILED;
     }
@@ -1290,19 +1288,12 @@ void site_flatlist(FILE *f, struct site *site)
     fprintf(f, "siteend|%s\n", site->local_is_different?"changed":"unchanged");
 }
 
-static void progress_cb(void *userdata, off_t progress, off_t total)
+void site_sock_progress_cb(void *userdata, off_t progress, off_t total)
 {
     fe_transfer_progress(progress, total);
 }
 
-static void notify_cb(void *userdata, sock_status status, const char *info)
-{
-    fe_connection(status, info);
-}
-
 void fe_initialize(void)
 {
-    sock_register_notify(notify_cb, NULL);
-    sock_register_progress(progress_cb, NULL);
-    sock_init();
+    ne_sock_init();
 }
