@@ -1,6 +1,6 @@
 /* 
    sitecopy, for managing remote web sites.
-   Copyright (C) 1999-2000, Joe Orton <joe@orton.demon.co.uk>
+   Copyright (C) 1999-2000, Joe Orton <joe@manyfish.co.uk>
                                                                      
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -292,7 +292,7 @@ enum site_protocol_modes {
     siteproto_ftp,
     siteproto_dav,
     siteproto_rsh,
-    siteproto_local, 
+    siteproto_local,
     siteproto_unknown
 };
 
@@ -349,10 +349,7 @@ struct site {
     struct site_host proxy;
 
     enum site_protocol_modes protocol;
-    char *proto_string; /* If protocol == siteproto_unknown,
-			 * proto_string is set to the string used
-			 * in the rcfile to identify the protocol.
-			 */
+    char *proto_string; /* protocol name used in rcfile. */
     const struct proto_driver *driver; /* the protocol driver routines */
 
     char *remote_root; /* root directory of site on server */
@@ -363,6 +360,7 @@ struct site {
     unsigned int local_isrel:1; /* is the local root directory relative to home dir */
 
     char *infofile;  /* local storage file in ~/.sitecopy/  */
+    char *certfile;  /* file in which cached SSL certificate is stored. */
     FILE *storage_file;  /* The file opened for the storage file */
 
     /* Options for the site */
@@ -373,9 +371,11 @@ struct site {
     unsigned int ftp_pasv_mode:1;
     unsigned int ftp_echo_quit:1;
     unsigned int ftp_forcecd:1;
+    unsigned int ftp_use_cwd:1;
     unsigned int http_use_expect:1;
     unsigned int http_limit:1;
     unsigned int http_secure:1;
+    unsigned int http_tolerant:1;
     char *rsh_cmd;
     char *rcp_cmd;
 
@@ -480,8 +480,16 @@ void site_initialize(struct site *);
 /* Catch up the site - mark all files as updated remotely */
 void site_catchup(struct site *site);
 
-/* Unimplemented */
-int site_verify(struct site *site);
+/* Verify that that the stored state of the remote site matches the
+ * actual make up of the remote site. Returns:
+ *    SITE_OK       if states match up
+ *    SITE_ERRORS   if states do not match
+ *    SITE_FAILED   if the comparison could not begin (e.g. auth failure).
+ *
+ * If SITE_ERRORS is returned, then *numremoved is set to the number
+ * of files missing from the server, and fe_verified() will have been
+ * called for any changed or added to the remote site.  */
+int site_verify(struct site *site, int *numremoved);
 
 /* Update the remote site.
  * fe_updating, fe_updated, fe_setting_perms, fe_set_perms may be
